@@ -150,7 +150,7 @@ header_update = function(header, data)
 #' Extra Bytes Attributes Tools
 #'
 #' Functions that update a header to describe Extra Bytes Attributes according to the
-#' \href{http://www.asprs.org/wp-content/uploads/2019/07/LAS_1_4_r15.pdf}{LAS specifications}
+#' \href{https://www.asprs.org/wp-content/uploads/2019/07/LAS_1_4_r15.pdf}{LAS specifications}
 #'
 #' @param header list
 #' @param name character. The name of the extrabytes attributes to add in the file.
@@ -303,7 +303,7 @@ header_add_extrabytes_manual = function(header, name, desc, type, offset = NULL,
 #' Coordinate Reference System Tools
 #'
 #' Functions that update a header to describe coordinates reference system according to the
-#' \href{http://www.asprs.org/wp-content/uploads/2019/07/LAS_1_4_r15.pdf}{LAS specifications}
+#' \href{https://www.asprs.org/wp-content/uploads/2019/07/LAS_1_4_r15.pdf}{LAS specifications}
 #'
 #' @param header list
 #' @param epsg integer. An EPSG code
@@ -342,7 +342,7 @@ header_set_epsg = function(header, epsg)
     }
     else
     {
-      pos <- length(header[["Variable Length Records"]][["GeoKeyDirectoryTag"]]) + 1
+      pos <- length(header[["Variable Length Records"]][["GeoKeyDirectoryTag"]][["tags"]]) + 1
     }
   }
 
